@@ -3,7 +3,7 @@
 #' @inheritParams archive_write
 #' @returns An 'archive_read' connection to the file within the archive to be read.
 #' @examples
-#' a <- archive(system.file(package = "archive", "extdata", "data.zip"))
+#' a <- system.file(package = "archive", "extdata", "data.zip")
 #' # Show files in archive
 #' a
 #'
@@ -20,18 +20,16 @@
 #' read.csv(archive_read(a, format = "zip"), nrows = 3)
 #' @export
 archive_read <- function(archive, file = 1L, mode = "r", format = NULL, filter = NULL, options = character()) {
-  archive <- as_archive(archive, options)
-  if (is_number(file)) {
-    file <- archive$path[[file]]
-  }
-
   assert("`file` must be a length one character vector or numeric",
     length(file) == 1 && (is.character(file) || is.numeric(file)))
 
-  assert(paste0("`file` {file} not found in `archive` {archive}"),
-    file %in% archive$path)
-
   options <- validate_options(options)
 
-  archive_read_(attr(archive, "path"), file, mode, archive_formats()[format], archive_filters()[filter], options, sz = 2^14)
+  if (!inherits(archive, "connection")) {
+    archive <- file(archive, "rb")
+  }
+
+  description <- glue::glue("archive_read({desc})[{file}]", desc = summary(archive)$description)
+
+  archive_read_(archive, file, description, mode, archive_formats()[format], archive_filters()[filter], options, sz = 2^14)
 }
